<?php

namespace NotificationXPro\Core;

use NotificationX\GetInstance;

class CSV_Handler {

    use GetInstance;

    public function __construct()
    {
        add_filter('nx_add_csv_data_line', [$this, 'csv_data_line'], 10, 6);
        add_filter( 'intermediate_image_sizes_advanced', '__return_empty_array', 10, 2 );
    }

    public function csv_data_line($return, $lines, $image_key, $slugged_headers, $headers, $uploadImage) {
        foreach ($lines as $line) {
            if (trim($line) !== '') {
                $row      = str_getcsv($line, ",", '"', "\\");
                $row_data = array_combine($slugged_headers, $row);
                 // Generate a unique index if it does not exist
                if (!isset($row_data['index']) || empty($row_data['index'])) {
                    $row_data['index'] = $this->generate_unique_index();
                }

                if ($image_key !== false && !empty($row[$image_key]) && $uploadImage) {
                    $image_url   = $row[$image_key];
                    $upload_data = Helper::upload_image_to_media_library($image_url);
                    if ($upload_data !== false) {
                        $row_data['image'] = [
                            'id'    => $upload_data['id'],
                            'title' => $upload_data['title'],
                            'url'   => $upload_data['url']
                        ];
                    }
                }
                $return[] = $row_data;
            }
        }
        $return = apply_filters('nx_csv_data_header_lines', $return, $lines, $headers);
        return $return;
    }

    private function generate_unique_index() {
        return sprintf(
            '%04x%04x-%04x-%04x-%04x-%04x%04x%04x',
            mt_rand(0, 0xffff), mt_rand(0, 0xffff),
            mt_rand(0, 0xffff),
            mt_rand(0, 0x0fff) | 0x4000,
            mt_rand(0, 0x3fff) | 0x8000,
            mt_rand(0, 0xffff), mt_rand(0, 0xffff), mt_rand(0, 0xffff)
        );
    }

    public function init($file, $uploadImage, $take = null, $chunkIndex = 0, $totalChunks = 1, $chunkSize = 10) {
        try {
            set_time_limit(0);
            $csv_content = file_get_contents($file);
            if ($csv_content === false) {
                return wp_send_json_error([
                    'error' => __('Unable to read the CSV file.', 'notificationx'),
                    'message' => __('Please check the CSV file and try again.', 'notificationx')
                ], 512);
            }
    
            $data = [];
            $lines = explode("\n", $csv_content);
    
            // Extract headers and process lines
            $headers = str_getcsv(array_shift($lines), ",", '"', "\\");
            $slugged_headers = array_map([$this, 'make_slug'], $headers);
            $image_key = array_search('image', $slugged_headers);
    
            // Slice lines for the current chunk
            $offset = $chunkIndex * $chunkSize + 1;
            $chunkLines = array_slice($lines, $offset, $chunkSize);
    
            // Include headers in each chunk for consistency
            if ($chunkIndex === 0) {
                array_unshift($chunkLines, implode(',', $headers)); // Add headers to the first chunk
            }
    
            // Process CSV lines chunk
            $data = apply_filters('nx_add_csv_data_line', [], $chunkLines, $image_key, $slugged_headers, $headers, $uploadImage);
    
            return wp_send_json_success([
                'success'     => true,
                'data'        => $data,
                'headers'     => $slugged_headers,
                'chunkIndex'  => $chunkIndex,
                'totalChunks' => $totalChunks
            ]);
        } catch (\Throwable $th) {
            return wp_send_json_error([
                'error' => __('Invalid CSV File', 'notificationx'),
                'message' => __('Please upload a valid CSV file.', 'notificationx')
            ], 512);
        }
    }
    

    public function make_slug($text) {
        $text = preg_replace('~[^\pL\d]+~u', '_', $text);
        $text = iconv('utf-8', 'us-ascii//TRANSLIT', $text);
        $text = preg_replace('~[^_\w]+~', '', $text);
        $text = trim($text, '_');
        $text = preg_replace('~_+~', '_', $text);
        $text = strtolower($text);
        if (empty($text)) {
            return 'n_a';
        }
        return $text;
    }

}